/*******************************************************************************
* Title                 :   Debug Application   
* Filename              :   debug_app.c
* Author                :   Jacob Beningo
* Origin Date           :   07/01/2015
* Version               :   1.0.0
* Compiler              :   Generic
* Target                :   Generic
* Copyright             :   Beningo Engineering
*
* THIS SOFTWARE IS PROVIDED BY BENINGO ENGINEERING "AS IS" AND ANY EXPRESSED
* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL BENINGO ENGINEERING OR ITS CONTRIBUTORS BE LIABLE FOR ANY
* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
* THE POSSIBILITY OF SUCH DAMAGE.
*
*******************************************************************************/
/****************************************************************************
* Doxygen C Template
* Copyright (c) 2013 - Jacob Beningo - All Rights Reserved
*
* Feel free to use this Doxygen Code Template at your own risk for your own
* purposes.  The latest license and updates for this Doxygen C template can be  
* found at www.beningo.com or by contacting Jacob at jacob@beningo.com.
*
* For updates, free software, training and to stay up to date on the latest 
* embedded software techniques sign-up for Jacobs newsletter at 
* http://www.beningo.com/814-2/
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Template. 
*
*****************************************************************************/
/*************** SOURCE REVISION LOG *****************************************
*
*    Date    Version   Author         Description 
*  07/01/15   1.0.0   Jacob Beningo   Initial Release.
*
*******************************************************************************/
/** @file debug_app.c
 *  @brief This is the source file for the serial debug interface application
 */
/******************************************************************************
* Includes
*******************************************************************************/
#include <stdint.h>				  /* For portable types */
#include "debug_app.h"			/* For Debug definitions */		
#include "cbuf.h"           /* For a public domain circular buffer */
#include "uart.h"           /* For uart driver definitions */
#include <stdarg.h>         /* For variable arguemtsn */
#include <stdio.h>          /* For sprintf formatting */

/******************************************************************************
* Module Preprocessor Constants
*******************************************************************************/


/******************************************************************************
* Module Preprocessor Macros
*******************************************************************************/

/******************************************************************************
* Module Typedefs
*******************************************************************************/

/******************************************************************************
* Module Variable Definitions
*******************************************************************************/
/**
 * Stores a single character read from the receive uart peripheral
 */
static uint8_t volatile RxChar = 0;

/**
 * Stores a single character that will transmitted over the uart.
 */
static uint8_t volatile TxChar = 0;

/**
 * Defines a variable that is used as a circular buffer for incoming uart data
 */
static myQ volatile RxBuffer;

/**
 * Defines a variable that is used as a circular buffer for outgoing uart data
 */
static myQ volatile TxBuffer;

/******************************************************************************
* Function Prototypes
*******************************************************************************/

/******************************************************************************
* Function Definitions
*******************************************************************************/
/******************************************************************************
* Function : Debug_Init()
*//** 
* \b Description:
*
*  This function is used to initialize the debug application buffers
*
* @return 		void
*
* @see Debug_Init
*
* \b Example Example:
* @code
*    Debug_Init();
* @endcode
*
*  ----------------------
*  - HISTORY OF CHANGES -
*  ----------------------
*    Date    Version   Author         Description 
*  07/01/15   1.1.0   Jacob Beningo   Initial Release.
*
*******************************************************************************/
void Debug_Init(void)
{
    CBUF_Init(RxBuffer);
    CBUF_Init(TxBuffer);
}


/******************************************************************************
* Function : Debug_Process()
*//**
* \b  Description:
*
*  This function determines if there is enough data in the RxBuffer to start
*  processing the data stream.
*
* @see Debug_Init
* @see Debug_Process
* @see Debug_printf
* @see Debug_IsrRx
* @see Debug_IsrTx
*
* \b Example:
* @code
*    None
* @endcode
*
*  ----------------------
*  - HISTORY OF CHANGES -
*  ----------------------
*    Date    Software Version    Initials   Description 		<br>
*  07/01/15       1.1.0            JWB      Function Implemented.	<br>
*
*******************************************************************************/
void Debug_Process(void)
{
    uint16_t NumRxBytes;
    uint16_t NumRxBytesExpected; 

    // How many bytes have been received?
    NumRxBytes = CBUF_Len(RxBuffer);
    
    /*
     * This is where custom code is specified.  For this example, 
     * the first index specifies the number of bytes to be received
     * If at least that number have been stored then process the packet
     */
    if(NumRxBytes > 0) 
    { 
      /* Assemble the 16 bit packet size */
    	NumRxBytesExpected = CBUF_Get(RxBuffer, 1) << 8;
    	NumRxBytesExpected |= CBUF_Get(RxBuffer,2);

    	if(NumRxBytes >= NumRxBytesExpected)
    	{
         /* 
          * This is where something interesting would be done with the 
          * packet data.
          */	    
  		}
    }
}

/******************************************************************************
* Function : Debug_IsrRx()
*//**
* \b  Description:
*
*  This function fires when byte data is received on the UART.  The interrupt
*  verifies that the data is for the Debug uart, retrieves the data byte and
*  stores the data on a circular buffer.
*
* @see Debug_Init
* @see Debug_Process
* @see Debug_printf
* @see Debug_IsrRx
* @see Debug_IsrTx
*
* \b Example:
* @code
*    None
* @endcode
*
*  ----------------------
*  - HISTORY OF CHANGES -
*  ----------------------
*    Date    Software Version    Initials   Description 		<br>
*  07/01/15       1.0.0            JWB      Function Implemented.	<br>
*
*******************************************************************************/
void Debug_IsrRx (void)
{
    uint8_t RxChar

    if(Uart_IsDataPresent(UART_1) == TRUE)
    {
      RxChar = Uart_GetChar(UART_1);

      CBUF_Push(RxBuffer, RxChar);
    }   
}

/******************************************************************************
* Function : Debug_IsrTx()
*//**
* \b  Description:
*
*  This function fires when byte data is transmitted on the UART.  The interrupt
*  verifies that the data is for the Debug uart, retrieves the data byte and
*  transmits it from the circular buffer.
*
* @see Debug_Init
* @see Debug_Process
* @see Debug_printf
* @see Debug_IsrRx
* @see Debug_IsrTx
*
* \b Example:
* @code
*    None
* @endcode
*
*  ----------------------
*  - HISTORY OF CHANGES -
*  ----------------------
*    Date    Software Version    Initials   Description 		<br>
*  07/01/15       1.0.0            JWB      Function Implemented.	<br>
*
*******************************************************************************/
void Debug_IsrTx (void)
{
    uint8_t TxChar;

    Uart_TxIFClear(UART_1);

    /* If the buffer isn't empty then transmit the next byte */
    if(CBUF_IsEmpty(TxBuffer) == FALSE)
    {
    	TxChar = CBUF_Pop(TxBuffer);

	    Uart_PutChar(UART_1, TxChar);   	
    }
}

/******************************************************************************
* Function : Debug_printf()
*//**
* \b  Description:
*
*  This function is used to format a printf string into a buffer that can be
*  added to the circular buffer and transmitted out the Debug port.
*
* @see Debug_Init
* @see Debug_Process
* @see Debug_printf
* @see Debug_IsrRx
* @see Debug_IsrTx
*
* \b Example:
* @code
*    Debug_printf("There are %d commands in the queue\n\r", CommandsQueued);
* @endcode
*
*  ----------------------
*  - HISTORY OF CHANGES -
*  ----------------------
*    Date    Software Version    Initials   Description 		<br>
*  07/01/15       1.0.0            JWB      Function Implemented.	<br>
*
*******************************************************************************/
void Debug_printf(const char *Format, ...)
{
	char Buffer[PRINTF_BUFFER_SIZE];
  va_list Args;
  uint8_t Length;
  uint8_t TxChar;
  uint8_t i = 0;

  /*
   * Create a formatted buffer based on the arguments and the character pointer
   * supplied to this function.  Buffer will contain the formatted character
   * buffer that can be sent to the serial device.
   */
	va_start(Args, Format);
	vsprintf(Buffer, Format, Args);
	va_end(Args);

  /* Determine the length of the final formatted string */
  Length = strlen(Buffer);

  /* Move the formatted string buffer to the circular buffer */
  for(i = 0; i < Length; i++)
  {
     CBUF_Push(TxBuffer, Buffer[i]);
  }

  /* 
   * Get the first character and begin transmission.  
   * Interrupts will transmit the remaining characters
   */
  TxChar = CBUF_Pop(TxBuffer);
  Uart_PutChar(UART_1, TxChar);
}

/*************** END OF FUNCTIONS ***************************************************************************/
